-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\_setbspriority.sql
-- SID: 1.5
-- Date Modified: 04/10/21 14:56:22
--
-- Module Description
--   Defines the _SetBSPriority stored procedure
USE Object
GO

---------------------------------------------------------------------------------------------------
-- _SetBSPriority
PRINT 'Creating procedure _SetBSPriority'

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_SetBSPriority' AND type = 'P')
  DROP PROCEDURE _SetBSPriority
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\_setbspriority.sql
-- SID: 1.5
-- Date Modified: 04/10/21 14:56:22
-- Implementation of the SetBSPriority method.
--
--  Caller may optionally provide a temporary table called #Instances which conforms
--  to the usual conventions for instance tables. Every object in this table will
--  have its priority set to the new value specified by @NewPriority, or when the #NewPriority
--  is provided, to the values in that table based on joining the rCorr values from 
--  the #Instances and #NewPriority tables.
-- 
--  If supplied, the #NewPriority table must have the following definition:

--  CREATE TABLE #NewPriority
--  (
--    rCorr       INT 	   NOT NULL,
--    newValue    TINYINT  NULL,
--    error       INT 	   NULL
--  )

-- The rCorr column correlates to the same column in the #Instances table.

-- The error column is used on output to indicate whether the newValue value is
-- valid. Upon return this value will be 0 for valid entries or one of the error
-- codes (see below) if invalid.
  
-- Return values:
--  >0        Number of objects successfully updated. Priority is only set if old & new values are different.
--  -30000    Unknown error updating object(s).
--  -30001    One or more parameters are invalid.
          
-- Error Messages:
--   The following error messages will be raised if the corresponding error occurs:
--   Error  Description
--   51200  Specified object does not exist
--   51201  Unknown bit set in ExecMode
--   51501  GTMPT1501E: Priority must be 1 (Critical) or 2 (High) or 3 (Medium) or 4 (Low) or
--                      5 (Ignore) or 6 (InheritFromEvent)
--   51502  GTMPT1502E: The specified CID must be a Business System (LOB).
--   64002  GTMTU1032W: Request was ignored because a newer request was available.
CREATE PROCEDURE _SetBSPriority
  @cid         ClassID,
  @id          ObjID,
  @NewPriority TINYINT = NULL, -- New Priority to assign to the object(s). If specified, must be 1-6
                               -- If not specified then #Instances table is required.
  @ExecMode    TINYINT = 0     -- Execution Mode. Bitmask controlling the execution of the method. The currently
                               -- defined bit values are:
                               --   1   ValidateParams. Parameters are validated and no other action is performed.
                               --   2   BestEffort. Attempts to ignore errors on individual rows in the input table
                               --       and do as much work as possible.
                               --   4   ReportErrors. Set the  'error' column in the input table to indicate
                               --       which error occurred on each row.
AS
  SET NOCOUNT ON

  IF @ExecMode | 7 != 7
  BEGIN
    RAISERROR(51201, 4, 1) WITH NOWAIT  -- Unknown bit set in ExecMode
    RETURN 51201
  END

  -- Map @ExecMode bits to individual variables for cleaner looking comparisons later
  DECLARE @ValidateParams TINYINT, @BestEffort TINYINT, @ReportErrors TINYINT
  SET @ValidateParams = @ExecMode & 1
  SET @BestEffort     = (@ExecMode & 2) / 2
  SET @ReportErrors   = (@ExecMode & 4) / 4

  DECLARE @err INT, @rc INT, @ret INT, @sql NVARCHAR(1000)
  SET @ret = 0

  DECLARE @tblid INT
  SET @tblid = OBJECT_ID('tempdb..#Instances')
  IF @tblid IS NULL
  BEGIN
    CREATE TABLE #Instances
    (
      cid       ClassID   NULL,
      id        ObjID     NULL,
      rCorr     INT       NOT NULL,
      errorId   INT       NULL
    )
    INSERT INTO #Instances ( cid, id, rCorr, errorId ) VALUES ( @cid, @id, 0, 0 )
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  END
  ELSE
  BEGIN
    -- Map instances to their cid, id values
    IF EXISTS (SELECT 1 FROM #Instances WHERE cid IS NULL OR id IS NULL)
      EXEC @rc = resolveCidId
    -- Make sure all instances got mapped
    IF EXISTS (SELECT 1 FROM #Instances WHERE cid IS NULL OR id IS NULL)
    BEGIN
      RAISERROR(51200, 4, 1) WITH NOWAIT
      UPDATE #Instances SET errorId = 51200 WHERE cid IS NULL OR id IS NULL
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      SET @ret = -30001
      IF @BestEffort = 0
        GOTO EndOfProc
    END
  END

  UPDATE #Instances SET errorId = 0 WHERE cid IS NOT NULL AND id IS NOT NULL

  -- Make sure class=LOB as this proc only sets LOB object priority
  SELECT @rc = COUNT(*) FROM #Instances WHERE cid <> 'LOB'
  IF @rc > 0
  BEGIN
    RAISERROR(51502, 4, 1) WITH NOWAIT
    UPDATE #Instances SET errorId = 51502 WHERE cid <> 'LOB'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    SET @ret = -30001
    IF @BestEffort = 0
      GOTO EndOfProc
  END

  -- If we have multiple settings for a resource, use the latest setting
  IF EXISTS(SELECT cid, id, COUNT(*) FROM #Instances GROUP BY cid, id HAVING COUNT(*) > 1)
  BEGIN
    UPDATE i2 SET errorId = 64002 FROM #Instances i2
      WHERE rCorr < (
        SELECT MAX(rCorr) FROM #Instances i1
          WHERE i2.cid = i1.cid AND i2.id = i1.id
          GROUP BY i1.cid, i1.id
      )
  END

  -- Check for optional #NewPriority table
  SET @tblid = OBJECT_ID('tempdb..#NewPriority')
  IF @tblid IS NULL
  BEGIN
    CREATE TABLE #NewPriority
    (
      rCorr       INT 	   NOT NULL,
      newValue    TINYINT  NULL,
      error       INT 	   NULL
    )
    INSERT INTO #NewPriority ( rCorr, newValue, error ) VALUES ( 0, @NewPriority, NULL )
    UPDATE #Instances SET rCorr = 0
  END

  -- Make sure no invalid Priorities were specified
  UPDATE #NewPriority SET error = 0

  SELECT @rc = COUNT(*) FROM #NewPriority WHERE newValue IS NULL OR newValue < 1 OR newValue > 6
  IF @rc > 0
  BEGIN
    RAISERROR(51501, 4, 1) WITH NOWAIT
    UPDATE #NewPriority SET error = 51501 WHERE newValue IS NULL OR newValue < 1 OR newValue > 6
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    SET @ret = -30001
    IF @BestEffort = 0
      GOTO EndOfProc
  END

  -- Unless we're just validating parameters, go ahead and do the updates now
  IF @ValidateParams = 0
  BEGIN  
    DECLARE @tcount INT
    SET @tcount = @@TRANCOUNT
    IF @tcount = 0
      BEGIN TRANSACTION _SetBSPriority
    ELSE
      SAVE TRANSACTION _SetBSPriority

    INSERT INTO StagedBSPriority (cid, id, priority)
      SELECT i.cid, i.id, n.newValue
    FROM LineOfBusiness_V AS lob 
      JOIN #Instances   AS i ON i.cid = lob.cid AND i.id = lob.id 
      JOIN #NewPriority AS n ON n.rCorr = i.rCorr 
      WHERE i.errorId = 0 AND n.error = 0 AND lob._PriorityID <> n.newValue

    SELECT @err = @@ERROR, @rc = @@ROWCOUNT    
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION _SetBSPriority
      RETURN -30000
    END
    SET @ret = @rc

    -- Insert dummy record to StagedLOB so SEL processes BSPriority all the time
    IF @rc > 0 AND NOT EXISTS (SELECT 1 FROM StagedLOB WHERE dst_id = 0)
    BEGIN
      INSERT INTO StagedLOB ( link_type, src_cid, src_id, dst_cid, dst_id, ctime )
        SELECT 'LOBC', 'LOB', 0, 'LOB', 0, CURRENT_TIMESTAMP

      SELECT @err = @@ERROR, @rc = @@ROWCOUNT    
      IF @err <> 0
      BEGIN
        ROLLBACK TRANSACTION _SetBSPriority
        RETURN -30000
      END
    END

    IF @tcount = 0
      COMMIT TRANSACTION _SetBSPriority
  END

EndOfProc:
  RETURN @ret
GO
