-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File:archive\db\sql\procs\asisp_loadsetbspriority.sql
-- SID: 1.2
-- Date Modified: 04/10/22 10:09:57
-- 
-- Module Description
--   Loads parameters required for _SetBSPriority stored procedure bulk processing
--
USE Object
GO

-------------------------------------------------------------------------------
-- asisp_loadSetBSPriority
-------------------------------------------------------------------------------
PRINT 'Creating procedure asisp_loadSetBSPriority'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_loadSetBSPriority' AND type = 'P')
  DROP PROCEDURE asisp_loadSetBSPriority
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File:archive\db\sql\procs\asisp_loadsetbspriority.sql
-- SID: 1.2
-- Date Modified: 04/10/22 10:09:57
--
-- This is invoked by SetBSPriority.ksh script, CommonListener, or ABS to invoke 
-- _SetBSPriority stored proc. Output can be formated into different result sets based on 
-- @outformat variable. The resource and priority parameters are provided via #Instances
-- and #NewPriority tables described below.
--
-- #NewPriority provides the priority parameters:
--
-- CREATE TABLE #BSPriorityParms
-- (
--   rCorr     INT            NULL,
--   newValue  NVARCHAR(32)   NULL,
--   error     INT            NULL
-- )
--
-- The corresponding instances are provided in #Instances table which is defined
-- by resolveCidId proc.  The rCorr correlates the instance to a particular
-- #BSPriorityParms (i.e., priority setting).
--
-- CREATE TABLE #Instances (
--   cid        ClassID       NULL,
--   id         ObjID         NULL,
--   rCorr      INT           NULL,             
--   className  NVARCHAR(24)  NULL,
--   prodVer    NVARCHAR(85)  NULL,     
--   instid     NVARCHAR(412) NULL,
--   uinstid    NVARCHAR(412) NULL,
--   objPath    NVARCHAR(255) NULL,
--   maxRes     INT           NULL,
--   errorId    INT           NULL
-- ) 
--  maxRes: Flag associated with Object path parm. Default 1
--    0=Object path resolves to single resouce.
--    1=Object path parm can resolve to one LOB.
--    2=Object path parm can resolve to multiple LOBs.
--    3=Object path parm can resolve to multiple resources.
--
-- Input parameters:
-- @resolveCidId = 1 (resolve cid/id in #Instances table using resolveCidId procedure)
--               = 0 (does not resolve cid/id in #Instances)
--
-- @ExecMode is passed through to _SetWeights method
--
-- @outformat = 0 (no output)
--            = 1 (output format used by CommonListener)
--            = 2 (output format used by CLI)
--            = 3 (output format used by ABS while method validation)
--
-- @verbose >= 1 will execute this procedure in verbose mode 

CREATE PROCEDURE asisp_loadSetBSPriority
  @resolveCidId INT = 1,
  @ExecMode     INT = 6,
  @outformat    TINYINT = 0, 
  @verbose      TINYINT = 0     
AS

  SET NOCOUNT ON

  DECLARE @now NVARCHAR(40), @err INT, @rc INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_loadSetBSPriority: @resolveCidId=%d, @ExecMode=%d, @outformat=%d, @verbose=%d', 0, 1, @now, @resolveCidId, @ExecMode, @outformat, @verbose) WITH NOWAIT
  END

  -- Bulk processing table for _SetBSPriority method
  CREATE TABLE #NewPriority
  (
    rCorr     INT     NOT NULL,
    newValue  TINYINT NOT NULL,
    error     INT     NULL
  )

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
     BEGIN TRANSACTION asisp_loadSetBSPriority
  ELSE
     SAVE TRANSACTION asisp_loadSetBSPriority

  -- The resolveCidId will resolve cid/id from the input data provided in #Instances table
  -- if needed. _SetBSPriority requires cid/id parameter. This is done when @resolveCidId
  -- is set to 1.  Not all invokers may need this (e.g., ABS may not have cid/ids).
  -- If errors occur during resolution, resolveCidId updates errorId.
  IF OBJECT_ID('tempdb..#Instances') IS NOT NULL
  BEGIN
    IF @resolveCidId = 1
    BEGIN
      EXEC resolveCidId
    END
  END

  -- Convert newValue (priority) to corresponding values required by _SetBSPriority 
  UPDATE #BSPriorityParms 
    SET 
     newValue   = CASE LOWER(newValue)
                    WHEN 'critical'            THEN '1'
                    WHEN 'c'                   THEN '1'
                    WHEN 'high'                THEN '2'
                    WHEN 'h'                   THEN '2'
                    WHEN 'medium'              THEN '3'
                    WHEN 'm'                   THEN '3'
                    WHEN 'low'                 THEN '4'
                    WHEN 'l'                   THEN '4'
                    WHEN 'ignore'              THEN '5'
                    WHEN 'i'                   THEN '5'
                    WHEN 'inheritfromevent'    THEN '6'
                    WHEN 'e'                   THEN '6'
                    ELSE newValue
                  END
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_loadSetBSPriority
    RETURN -30000
  END
  IF @verbose >= 1 AND @rc > 0
  BEGIN
    SET @now=CONVERT(NVARCHAR,CURRENT_TIMESTAMP,121)
    RAISERROR('%s: asisp_loadSetBSPriority: converted %d string parameter(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

   -- Set error in #BSPriorityParms if any of the above columns strings 
   -- cannot be resolved.
   UPDATE #BSPriorityParms SET error = 51501
     WHERE ISNUMERIC(newValue) = 0 
                       
  -- Populate #NewPriority using #BSPriorityParms table
  INSERT INTO #NewPriority (rCorr, newValue, error)
    SELECT bs.rCorr, bs.newValue, bs.error
      FROM #BSPriorityParms AS bs
      WHERE bs.error IS NULL OR bs.error = 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_loadSetBSPriority
    RETURN -30000
  END
  IF @verbose >= 1 AND @rc > 0
  BEGIN
    SET @now=CONVERT(NVARCHAR,CURRENT_TIMESTAMP,121)
    RAISERROR('%s: asisp_loadSetBSPriority: inserted %d rows into _SetBSPriority temp table', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Method call
  SET @rc = 0
  EXEC @rc = _SetBSPriority @cid=NULL, @id=NULL, @ExecMode=@ExecMode

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: After _SetBSPriority method call (rc=%d)', 0, 1, @now, @rc)
  END

  -- Turn off in case we miss a ISNULL check
  SET CONCAT_NULL_YIELDS_NULL OFF
  -- Select output result set based on @outformat variable
  IF @outformat = 1  -- For Common Listener
  BEGIN
    SELECT 
      rCorr, 
      sm.description + '(ErrorID = ' + CAST(i.errorId AS NVARCHAR(9)) + ') (' + 
      i.className + '/' + i.prodVer + ', ' + i.instid + '/' + i.objPath + ')'
    FROM #Instances i
    JOIN master..sysmessages sm ON sm.error = i.errorId
    WHERE i.errorId IS NOT NULL AND i.errorId <> 0
    UNION
    SELECT 
      b.rCorr, 
      sm.description + '(ErrorID = ' + CAST(b.error AS NVARCHAR(9)) + ') (' + 
      CAST(b.newValue AS NVARCHAR(9)) + ')' 
    FROM #BSPriorityParms b
    JOIN master..sysmessages sm ON sm.error = b.error
    WHERE b.error IS NOT NULL AND b.error <> 0
    UNION
    SELECT 
      rCorr, 
      'ErrorID = ' + CAST(error AS NVARCHAR(9)) + ' (priority=' +
      CAST(newValue AS NVARCHAR(9)) + ')' 
    FROM #NewPriority WHERE error IS NOT NULL AND error <> 0
  END
  ELSE IF @outformat = 2  -- For CLI 
  BEGIN
    IF NOT EXISTS (SELECT 1 FROM #BSPriorityParms WHERE error <> 0)
    BEGIN
      IF NOT EXISTS (SELECT 1 FROM #NewPriority WHERE error <> 0)
      BEGIN
        IF NOT EXISTS (SELECT 1 FROM #Instances WHERE errorId <> 0)
        BEGIN
          SELECT 'o-c-no errors'
          GOTO EndDOfProc
        END
      END
    END
      SELECT 'o-c-' + '|' +
             ISNULL(i.cid, '') + '|' +
             ISNULL(CAST(i.id AS NVARCHAR(9)), '') + '|' +
             ISNULL(i.className, '') + '|' + 
             ISNULL(i.prodVer, '') + '|' +
             ISNULL(i.instid, '') + '|' +
             ISNULL(i.uinstid, '') + '|' +
             ISNULL(i.objPath, '') + '|' +
             ISNULL(CAST(bs.error AS NVARCHAR(9)), '') + '|' +
             ISNULL(sm.description, '') + '|' +
             ISNULL(CAST(bs.newValue AS NVARCHAR(9)), '') + '|'
      FROM #BSPriorityParms AS bs
      JOIN #Instances AS i ON i.rCorr=bs.rCorr
      JOIN master..sysmessages AS sm ON (bs.error = sm.error)
      WHERE bs.error <> 0 AND bs.error IS NOT NULL
    UNION
      SELECT 'o-c-' + '|' +
             ISNULL(i.cid, '') + '|' +
             ISNULL(CAST(i.id AS NVARCHAR(9)), '') + '|' +
             ISNULL(i.className, '') + '|' + 
             ISNULL(i.prodVer, '') + '|' +
             ISNULL(i.instid, '') + '|' +
             ISNULL(i.uinstid, '') + '|' +
             ISNULL(i.objPath, '') + '|' +
             ISNULL(CAST(np.error AS NVARCHAR(9)), '') + '|' +
             ISNULL(sm.description, '') + '|' +
             ISNULL(CAST(np.newValue AS NVARCHAR(9)), '') + '|'
      FROM #NewPriority AS np
      JOIN #Instances AS i ON i.rCorr=np.rCorr
      JOIN master..sysmessages AS sm ON (np.error = sm.error)
      WHERE np.error <> 0 AND np.error IS NOT NULL
    UNION
      SELECT 'o-c-' + '|' +
             ISNULL(cid, '') + '|' +
             ISNULL(CAST(id AS NVARCHAR(9)), '') + '|' +
             ISNULL(className, '') + '|' + 
             ISNULL(prodVer, '') + '|' +
             ISNULL(instid, '') + '|' +
             ISNULL(uinstid, '') + '|' +
             ISNULL(objPath, '') + '|' +
             ISNULL(CAST(errorId AS NVARCHAR(9)), '') + '|' +
             ISNULL(sm.description, '') + '|' +
             '|'
      FROM #Instances
      JOIN master..sysmessages AS sm ON (errorId = sm.error)
      WHERE errorId <> 0 AND errorId IS NOT NULL
  END
  ELSE IF @outformat = 3  -- For ABS, set the error code in the input table 
  BEGIN
    UPDATE bs SET error = np.error FROM #NewPriority np
      JOIN #BSPriorityParms bs ON np.rCorr = bs.rCorr AND np.newValue = bs.newValue
      WHERE np.error IS NOT NULL AND np.error <> 0
  END

EndDOfProc:
  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION asisp_loadSetBSPriority

  -- Return code from _SetBSPriority
  RETURN @rc

GO
