-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\fixobjectstates.sql
-- SID: 1.2.4.4
-- Date Modified: 04/09/23 11:27:08
--
-- Module Description
--   Defines the FixObjectStates stored procedure
USE Object
GO

---------------------------------------------------------------------------------------------------
-- FixObjectStates
PRINT 'Creating procedure FixObjectStates'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'FixObjectStates' AND type = 'P')
  DROP PROCEDURE FixObjectStates
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\fixobjectstates.sql
-- SID: 1.2.4.4
-- Date Modified: 04/09/23 11:27:08
CREATE PROCEDURE FixObjectStates
  @max_objects  INT = 1000,   -- Maximum number of objects to fix
  @recompute    TINYINT = 1,  -- If 1 then the set of objects with discrepancies is recomputed,
                              -- otherwise the previous set is used
  @verbose      TINYINT = 0   -- If >= 1 then diagnostic messages are output 
AS
  SET NOCOUNT ON
  SET XACT_ABORT ON

  DECLARE @now NVARCHAR(40), @rc INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: FixObjectStates: starting', 0, 1, @now) WITH NOWAIT
  END

  DECLARE @starttime DATETIME
  SET @starttime = CURRENT_TIMESTAMP
  INSERT INTO FixObjectStatesLog ( starttime, max_objects ) VALUES ( @starttime, @max_objects )

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT

  IF @recompute = 1
  BEGIN
    -- Compute the set of objects whose exception counters do not match the exceptions in the DB
    EXEC ComputeExcpCountMismatches @exit_with_error = 0, @verbose = @verbose
    -- Compute the set of objects whose child event counters do not match the childevents in the DB
    EXEC ComputeChevCountMismatches @exit_with_error = 0, @verbose = @verbose
  END
  ELSE IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: FixObjectStates: using previously computed set of mismatches', 0, 1, @now) WITH NOWAIT
  END

  SELECT @rc = COUNT(*) FROM ExcpCountMismatch
  SELECT @rc = @rc + COUNT(*) FROM ChevCountMismatch
  UPDATE FixObjectStatesLog SET nmismatches = @rc WHERE starttime = @starttime
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: FixObjectStates: number of mismatches = %d', 0, 1, @now, @rc) WITH NOWAIT
  END
  IF @rc = 0
  BEGIN
    UPDATE FixObjectStatesLog SET endtime = CURRENT_TIMESTAMP WHERE starttime = @starttime

    -- Update SyncEvent record if it exists
    IF EXISTS (SELECT 1 FROM SyncEvent WHERE eventid = 'PauseStagedEventProcessing' AND eventno = 0)
    BEGIN
      UPDATE SyncEvent SET eventno = 1
      WHERE
            eventid = 'PauseStagedEventProcessing'
        AND eventno = 0
    END

    RETURN 0
  END

  CREATE TABLE #obj
  (
    cid ClassID NOT NULL,
    id  ObjID   NOT NULL
  )
  INSERT INTO #obj
    SELECT DISTINCT cid, id
    FROM ExcpCountMismatch
  INSERT INTO #obj
    SELECT DISTINCT cid, id
    FROM ChevCountMismatch AS m
    WHERE NOT EXISTS (SELECT * FROM #obj AS o WHERE o.cid = m.cid AND o.id = m.id)
  SELECT @rc = COUNT(*) FROM #obj
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: FixObjectStates: number of distinct objects with mismatches = %d', 0, 1, @now, @rc) WITH NOWAIT
  END
  UPDATE FixObjectStatesLog SET nobjects = @rc WHERE starttime = @starttime

  -- Set the value for the verbose flag to the individual asisp_fixObjectState calls.
  -- This is set to 1 only if our verbose flag is > 1
  DECLARE @iverbose TINYINT
  IF @verbose <= 1
    SET @iverbose = 0
  ELSE
    SET @iverbose = 1

  IF @max_objects > @rc
    SET @max_objects = @rc
  DECLARE fixobjectstates_c CURSOR FOR
    SELECT cid, id
    FROM #obj
    ORDER BY cid, id
  OPEN fixobjectstates_c
  DECLARE @cid ClassID, @id ObjID, @i INT
  SET @i = 1
  FETCH NEXT FROM fixobjectstates_c INTO @cid, @id
  WHILE @@FETCH_STATUS <> -1 AND @i <= @max_objects
  BEGIN
    IF @tcount = 0 AND @i % 10 = 1
      BEGIN TRANSACTION FixObjectStates

    IF @tcount = 0 OR @i = 1
    BEGIN
      -- Lock the Note_A table from updates for every 10 objects that are fixed if we're the
      IF @i % 10 = 1
      BEGIN

        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: FixObjectStates: %d/%d: locking the Note_A table', 0, 1, @now, @i, @max_objects) WITH NOWAIT
        END
        DECLARE @c INT
        SELECT @c = COUNT(*) FROM Note_A HOLDLOCK
      END
    END

    IF @verbose = 1 AND @i % 10 = 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: FixObjectStates: fixing object %d/%d', 0, 1, @now, @i, @max_objects) WITH NOWAIT
    END

    EXEC asisp_fixObjectState @cid, @id, @verbose = @iverbose

    UPDATE FixObjectStatesLog SET nfixed = @i, lastfixtime = CURRENT_TIMESTAMP WHERE starttime = @starttime

    IF @tcount = 0 AND @i % 10 = 1
      COMMIT TRANSACTION FixObjectStates

    SET @i = @i + 1
    FETCH NEXT FROM fixobjectstates_c INTO @cid, @id
  END
  DEALLOCATE fixobjectstates_c

  UPDATE FixObjectStatesLog SET endtime = CURRENT_TIMESTAMP WHERE starttime = @starttime
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: FixObjectStates: finished. Number of objects = %d', 0, 1, @now, @max_objects) WITH NOWAIT
  END

  -- Update SyncEvent record if it exists
  IF EXISTS (SELECT 1 FROM SyncEvent WHERE eventid = 'PauseStagedEventProcessing' AND eventno = 0)
  BEGIN
    UPDATE SyncEvent SET eventno = 1
    WHERE
          eventid = 'PauseStagedEventProcessing'
      AND eventno = 0
  END

  RETURN @max_objects
GO
